@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -58,9 +58,9 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Rory Toma, <rory@digeo.com>
- *  @author Martin Pala, <martin.pala@iol.cz>
+ *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: control.c,v 1.70 2003/10/25 19:17:10 hauk Exp $
+ *  @version \$Id: control.c,v 1.73 2004/02/06 20:02:20 martinp Exp $
  *
  *  @file
  */
@@ -77,7 +77,7 @@ static void  monitor_set(Service_T);
 static void  monitor_unset(Service_T);
 static void *wait_start(void *);
 static int   wait_stop(Service_T);
-static void  do_depend(Service_T, char *);
+static void  do_depend(Service_T, const char *);
 
 
 /* ------------------------------------------------------------------ Public */
@@ -87,7 +87,7 @@ static void  do_depend(Service_T, char *);
  * Handle the action for all processes in service list
  * @param action A string describing the action to execute
  */
-void control(char *action) {
+void control(const char *action) {
 
   Service_T s;
 
@@ -113,7 +113,7 @@ void control(char *action) {
  * @param G group name
  * @param action A string describing the action to execute
  */
-void control_group(char *G, char *action) {
+void control_group(const char *G, const char *action) {
 
   Service_T s;
 
@@ -142,7 +142,7 @@ void control_group(char *G, char *action) {
  * @param P process name
  * @param action A string describing the action to execute
  */
-void control_process(char *P, char *action) {
+void control_process(const char *P, const char *action) {
 
   Service_T s;
 
@@ -169,7 +169,7 @@ void control_process(char *P, char *action) {
  * @param P A service name as stated in the config file
  * @param action A string describing the action to execute
  */
-void d_check_service(char *P, char *action) {
+void d_check_service(const char *P, const char *action) {
 
   Socket_T s;
   char *auth= get_basic_authentication_header();
@@ -206,7 +206,7 @@ void d_check_service(char *P, char *action) {
  * @param action A string describing the action to execute
  * @return TRUE if the service was handled successfully otherwise FALSE
  */
-void check_service(char *P, char *action) {
+void check_service(const char *P, const char *action) {
 
   Service_T s= NULL;
 
@@ -489,7 +489,7 @@ static void monitor_unset(Service_T s) {
  * @param s A Service_T object
  * @param action An action to do on the dependant services
  */
-static void do_depend(Service_T s, char *action) {
+static void do_depend(Service_T s, const char *action) {
 
   Service_T child;
   
