@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -21,9 +21,9 @@
  *  System dependent device methods.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @author Martin Pala, <martin.pala@iol.cz>
+ *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_AIX.c,v 1.3 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: sysdep_AIX.c,v 1.6 2004/02/29 22:24:44 martinp Exp $
  *
  *  @file
  */
@@ -31,7 +31,7 @@
 #include <config.h>
 
 #include "monitor.h"
-#include "monit_device_sysdep.h"
+#include "device_sysdep.h"
 
 /**
  * Unknown OS special block device dummy mountpoint method.
