@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -68,7 +68,7 @@ static void set_environment(void);
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: env.c,v 1.19 2003/10/25 19:17:10 hauk Exp $
+ *  @version \$Id: env.c,v 1.24 2004/03/12 16:27:03 martinp Exp $
  * 
  *  @file
  */
@@ -82,20 +82,28 @@ static void set_environment(void);
  */
 void init_env() {
   
-  /*
-   * Do we have a valid tty?
-   */
-  Run.have_tty= (isatty(STDOUT_FILENO) && isatty(STDERR_FILENO));
-
-  /*
-   * Setup for safe(r) exec
-   */
+  /* Setup for safe(r) exec */
   set_sandbox();
 
-  /*
-   * Setup program environment
-   */
+  /* Setup program environment */
   set_environment();
+
+  /* Free program environment on exit */
+  atexit(destroy_env);
+  
+}
+
+
+/**
+ * Destroy the program environment
+ */
+void destroy_env() {
+  
+  FREE(Run.Env.cwd);
+  FREE(Run.Env.home);
+  FREE(Run.Env.user);
+  FREE(Run.localhostname);
+  FREE(Run.controlfile);
   
 }
 
@@ -204,7 +212,7 @@ static void set_environment(void) {
    * Get the localhost name
    */
   if(gethostname(localhost, sizeof(localhost)) < 0) {
-    snprintf(localhost, STRLEN, LOCALHOST);
+    snprintf(localhost, STRLEN, "%s", LOCALHOST);
   }
 
   /*
