@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -101,7 +101,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: processor.c,v 1.31 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: processor.c,v 1.39 2004/02/18 22:31:42 chopp Exp $
  *
  *  @file
  */
@@ -127,6 +127,7 @@ static HttpRequest create_HttpRequest(Socket_T);
 static void internal_error(Socket_T, int, char *);
 static HttpResponse create_HttpResponse(Socket_T);
 static int is_authenticated(HttpRequest, HttpResponse);
+static int get_next_token(char *s, int *cursor, char **r);
 
 
 /* ------------------------------------------------------------------ Public */
@@ -337,7 +338,7 @@ void set_content_type(HttpResponse res, const char *mime) {
  * @param name Header name to lookup the value for
  * @return The value of the specified header, NULL if not found
  */
-char *get_header(HttpRequest req, const char *name) {
+const char *get_header(HttpRequest req, const char *name) {
 
   HttpHeader p;
 
@@ -345,7 +346,7 @@ char *get_header(HttpRequest req, const char *name) {
     
     if(!strcasecmp(p->name, name)) {
       
-	return xstrdup(p->value);
+	return (p->value);
 
     }
     
@@ -362,7 +363,7 @@ char *get_header(HttpRequest req, const char *name) {
  * @param name The request parameter key to lookup the value for
  * @return The value of the specified parameter, or NULL if not found
  */
-char *get_parameter(HttpRequest req, const char *name) {
+const char *get_parameter(HttpRequest req, const char *name) {
 
   HttpParameter p;
 
@@ -370,7 +371,7 @@ char *get_parameter(HttpRequest req, const char *name) {
     
     if(!strcasecmp(p->name, name)) {
       
-	return xstrdup(p->value);
+	return (p->value);
 
     }
     
@@ -511,38 +512,31 @@ const char *get_status_string(int status) {
  * them to the doXXX methods defined in a cervlet module.
  */
 static void do_service(Socket_T s) {
-
   volatile HttpResponse res= create_HttpResponse(s);
   volatile HttpRequest req= create_HttpRequest(s);
-
+  
   if(res && req) {
-
     if(is_authenticated(req, res)) {
-      
       if(IS(req->method, METHOD_GET)) {
 
 	Impl.doGet(req, res);
-	
-      }
-      else if(IS(req->method, METHOD_POST)) {
+
+      } else if(IS(req->method, METHOD_POST)) {
 
 	Impl.doPost(req, res);
-	
-      }
-      else {
+
+      } else {
 	
 	send_error(res, SC_NOT_IMPLEMENTED, "Method not implemented");
-	
-      }
 
+      }
     }
 
     send_response(res);
-    
+
   }
 
   done(req, res);
-  
 }
 
 
@@ -571,7 +565,7 @@ static char *get_date(char *result, int size) {
  */
 static char *get_server(char *result, int size) {
 
-  snprintf(result, size, "%s %s", SERVER_NAME, SERVER_VERSION);
+  snprintf(result, size, "%s %s", SERVER_NAME, Run.httpdsig?SERVER_VERSION:"");
   
   return result;
   
@@ -599,7 +593,7 @@ static void send_response(HttpResponse res) {
 		 res->status_msg);
     socket_print(S, "Date: %s\r\n", date);
     socket_print(S, "Server: %s\r\n", server);
-    socket_print(S, "Content-length: %d\r\n", res->bufused);
+    socket_print(S, "Content-Length: %d\r\n", res->bufused);
     socket_print(S, "Connection: close\r\n");
     if(headers)
 	socket_print(S, "%s", headers);
@@ -624,17 +618,20 @@ static void send_response(HttpResponse res) {
 static HttpRequest create_HttpRequest(Socket_T S) {
 
   HttpRequest req;
-  char method[STRLEN];
   char url[REQ_STRLEN];
   char line[REQ_STRLEN];
   char protocol[STRLEN]; 
+  char method[REQ_STRLEN];
 
     
   if(socket_readln(S, line, REQ_STRLEN) == NULL) {
     internal_error(S, SC_BAD_REQUEST, "No request found");
     return NULL;
   }
-  
+
+  chomp(line,REQ_STRLEN);
+
+  /* RATS: ignore */ /* chomp does zero termination */
   if(sscanf(line, "%s %s HTTP/%3[1.0]", method, url, protocol) != 3) {
     internal_error(S, SC_BAD_REQUEST, "Cannot parse request");
     return NULL;
@@ -647,8 +644,7 @@ static HttpRequest create_HttpRequest(Socket_T S) {
 
   NEW(req);
   req->S= S;
-  chomp(protocol);
-  unescape_url(url);
+  url_decode(url);
   req->url= xstrdup(url);
   req->method= xstrdup(method);
   req->protocol= xstrdup(protocol); 
@@ -719,7 +715,8 @@ static void create_headers(HttpRequest req) {
       *value++= '\0';
       trim(line);
       trim(value);
-      chomp(value);
+      /* ATTENTION: Please double check this calculations: */
+      chomp(value, (REQ_STRLEN-(value-line)));
       header->name= xstrdup(line);
       header->value= xstrdup(value);
       
@@ -748,59 +745,38 @@ static void create_headers(HttpRequest req) {
  */
 static int create_parameters(HttpRequest req) {
 
-  int alloc= FALSE;
-  char *query_string= NULL;
-
-  if(IS(req->method, METHOD_POST)) {
+  char query_string[REQ_STRLEN]= {0};
 
+  if(IS(req->method, METHOD_POST) && get_header(req, "Content-Length")) {
+    int n;
+    int len; 
     Socket_T S= req->S;
-    
-    int len; char *l= get_header(req, "Content-Length");
-    
-    if(l && (len= atoi(l))) {
-      
-      query_string= xmalloc(sizeof(char) * (len + 1));
-      
-      if(socket_read(S, query_string, len) <= 0) {
-	
-	FREE(query_string);
-	return FALSE;
-	
-      }
-      
-      alloc= TRUE;
-      
+    if(1 != sscanf(get_header(req, "Content-Length"), "%d", &len)) {
+      return FALSE;
     }
-    
+    if(len < 0 || len > REQ_STRLEN)
+      return FALSE;
+    if(len==0)
+      return TRUE;
+    if(((n= socket_read(S, query_string, len)) <= 0) || (n != len)) {
+      return FALSE;
+    }
+    query_string[n]= 0;
   } else if(IS(req->method, METHOD_GET)) {
-    
     char *p;
-    
-    if( NULL != (p= strchr(req->url, '?'))) {
-      
-      *p++= '\0';
-      query_string= p;
-      
+    if(NULL != (p= strchr(req->url, '?'))) {
+      *p++= 0;
+      strncpy(query_string, p, REQ_STRLEN);
     }
-    
   }
-  
-  if(query_string) {
-    
+  if(*query_string) {
     char *p;
-    
-    if( NULL != (p= strchr(query_string, '/'))) {
-      
-      *p++= '\0';
-      req->pathinfo= p;
-      
+    if(NULL != (p= strchr(query_string, '/'))) {
+      *p++= 0;
+      req->pathinfo= xstrdup(p);
     }
-    
     req->params= parse_parameters(query_string);
-    
   }
-
-  if(alloc)FREE(query_string);
   
   return TRUE;
   
@@ -842,7 +818,8 @@ static void destroy_HttpRequest(HttpRequest req) {
   if(req) {
     
     FREE(req->method);
-    FREE(req->url); /* req->pathinfo is freed with url */
+    FREE(req->url); 
+    FREE(req->pathinfo);
     FREE(req->protocol);
     FREE(req->remote_user);
     if(req->headers)destroy_entry(req->headers);
@@ -928,7 +905,7 @@ static int is_authenticated(HttpRequest req, HttpResponse res) {
 static int basic_authenticate(HttpRequest req) {
 
   int rv= FALSE;
-  char *credentials= get_header(req, "Authorization");
+  const char *credentials= get_header(req, "Authorization");
 
   if(credentials) {
 
@@ -975,7 +952,6 @@ static int basic_authenticate(HttpRequest req) {
     }
     
     FREE(cr);
-    FREE(credentials);
 
   }
   
@@ -1024,25 +1000,67 @@ static void internal_error(Socket_T S, int status, char *msg) {
  */
 static HttpParameter parse_parameters(char *query_string) {
 
-  int controller= 1000;
+#define KEY 1
+#define VALUE 2
+
+  int token;
+  int cursor= 0;
+  char *key= NULL;
+  char *value= NULL;
   HttpParameter head= NULL;
 
-  while(query_string[0] && controller--) {
-    
-    HttpParameter p;
-    NEW(p);
-    p->value= makeword(query_string,'&');
-    p->name= makeword(p->value,'=');
-    plustospace(p->value);
-    unescape_url(p->value);
-    plustospace(p->name);
-    unescape_url(p->name);
-    p->next= head;
-    head= p;
-    
+  while((token= get_next_token(query_string, &cursor, &value))) {
+    if(token==KEY)
+      key= value;
+    else if(token==VALUE) {
+      HttpParameter p;
+      if(!key) goto error;
+      NEW(p);
+      p->name= key;
+      p->value= value;
+      p->next= head;
+      head= p;
+      key= NULL;
+    }
   }
 
   return head;
   
+ error:
+  FREE(key);
+  FREE(value);
+  destroy_entry(head);
+  
+  return NULL;
+  
 }
 
+
+/**
+ * A mini-scanner for tokenizing a query string
+ */
+static int get_next_token(char *s, int *cursor, char **r) {
+  
+  int i= *cursor;
+    
+  while(s[*cursor]) {
+    if(s[*cursor+1]=='=') {
+      *cursor+= 1;
+      *r= xstrndup(&s[i], (*cursor-i));
+      return KEY;
+    } 
+    if(s[*cursor]=='=') {
+      while(s[*cursor] && s[*cursor]!='&') *cursor+= 1;
+      if(s[*cursor]=='&') {
+	*r= xstrndup(&s[i+1], (*cursor-i)-1);
+	*cursor+= 1;
+      }  else
+	*r= xstrndup(&s[i+1], (*cursor-i));
+      return VALUE;
+    }
+    *cursor+= 1;
+  }
+
+  return FALSE;
+
+}
