@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -67,7 +67,7 @@
 #define MYPIDMASK          0122
 #define MYPIDDIR           PIDDIR
 #define MYPIDFILE          "monit.pid"
-#define MYPSTATEFILE       "monit.state"
+#define MYSTATEFILE        "monit.state"
 
 #define LOCALHOST          "localhost"
 #define SMTP_PORT          25
@@ -127,6 +127,9 @@
 #define UNIT_MEGABYTE      1048580
 #define UNIT_GIGABYTE      1073740000
 
+#define HASH_MD5           1
+#define HASH_SHA1          2
+#define DEFAULT_HASH       HASH_MD5
 
 /** ------------------------------------------------- Special purpose macros */
 
@@ -179,6 +182,20 @@ Sigfunc *signal(int signo, Sigfunc * func);
 
 /* --------------------------------------------------------- Data structures */
 
+/** Defines a mailinglist object */
+typedef struct mymail {
+  char *to;                         /**< Mail address for alert notification */
+  char *from;                                     /**< The mail from address */
+  char *subject;                                       /**< The mail subject */
+  char *message;                                       /**< The mail message */
+  char *opt_message;                 /**< An optional message used in alerts */
+  unsigned int events;  /*< Events for which this mail object should be sent */
+
+  /** For internal use */
+  struct mymail *next;                          /**< next recipient in chain */
+} *Mail_T;
+
+
 /** Defines a mail server address */
 typedef struct mymailserver {
   char *host;     /**< Server host address, may be a IP or a hostname string */
@@ -193,7 +210,23 @@ typedef struct myauthentication {
   int   digesttype;                      /**< How did we store the password */
   int   is_readonly;     /**< TRUE if this is a read-only authenticated user*/
   struct myauthentication *next;       /**< Next credential or NULL if last */
-} * Auth_T;
+} *Auth_T;
+
+
+/** Defines process tree */
+typedef struct myprocesstree {
+  int  pid;
+  int  ppid;
+  int  visited;
+  long mem_kbyte;
+  int  children_num;
+  int  children_sum;
+  long mem_kbyte_sum;
+
+  /** For internal use */
+  struct myprocesstree  *parent;
+  struct myprocesstree **children;
+} ProcessTree_T;
 
 
 /** Defines data for application runtime */
@@ -205,7 +238,6 @@ struct myrun {
   char *pidfile;                                  /**< This programs pidfile */
   char *statefile;                /**< The file with the saved runtime state */
   char *mygroup;                              /**< Group Name of the Service */
-  int  have_tty;                                 /**< Do we have a valid tty */
   int  debug;                   /**< Write debug information - TRUE or FALSE */
   int  use_syslog;                          /**< If TRUE write log to syslog */
   int  dolog;       /**< TRUE if program should log actions, otherwise FALSE */
@@ -218,6 +250,7 @@ struct myrun {
   char *httpsslclientpem;      /**< PEM file/dir to check against at connect */
   int  servicessl;                 /**< TRUE if monit has ssl service checks */
   int  allowselfcert;   /**< TRUE if self certified client certs are allowed */
+  int  httpdsig;   /**< TRUE if monit HTTP server presents version signature */
   int  httpdport;                    /**< The monit http server's portnumber */
   int  once;                                       /**< TRUE - run only once */
   int  init;                   /**< TRUE - don't background to run from init */
@@ -238,6 +271,7 @@ struct myrun {
   } Env;
 
   int mailserver_timeout;    /**< Connect and read timeout for a SMTP server */
+  Mail_T maillist;                /**< Global alert notification mailinglist */
   MailServer_T mailservers;    /**< List of MTAs used for alert notification */
   Auth_T credentials;    /** A list holding Basic Authentication information */
                                       /** User selected standard mail format */
@@ -271,8 +305,8 @@ typedef struct mycommand {
 
 /** Defines a protocol object with protocol functions */
 typedef struct myprotocol {
-  const char *name;                                     /**< Protocol name */
-  int(*check)(Socket_T);                /**< Service verification function */
+  const char *name;                                       /**< Protocol name */
+  int(*check)(Socket_T);                 /**< Protocol verification function */
 } *Protocol_T;
 
 
@@ -297,7 +331,8 @@ typedef struct myport {
   char *hostname;                                     /**< Hostname to check */
   int  port;                                                 /**< Portnumber */
   char *request;                              /**< Specific protocol request */
-  char *request_checksum; /**< The optional md5 checksum for a req. document */
+  char *request_checksum;     /**< The optional checksum for a req. document */
+  int  request_hashtype;  /**< The optional type of hash for a req. document */
   char *pathname;                   /**< Pathname, in case of an UNIX socket */
   char *address;               /**< Human readable destination of the socket */
   Generic_T generic;                                /**< Generic test handle */
@@ -345,20 +380,6 @@ typedef struct mydependant {
 } *Dependant_T;
 
 
-/** Defines a mailinglist object */
-typedef struct mymail {
-  char *to;                         /**< Mail address for alert notification */
-  char *from;                                     /**< The mail from address */
-  char *subject;                                       /**< The mail subject */
-  char *message;                                       /**< The mail message */
-  char *opt_message;                 /**< An optional message used in alerts */
-  unsigned int events;  /*< Events for which this mail object should be sent */
-
-  /** For internal use */
-  struct mymail *next;                          /**< next recipient in chain */
-} *Mail_T;
-
-
 /** Defines resource data */
 typedef struct myresource {
   int  resource_id;                              /**< Which value is checked */
@@ -410,7 +431,9 @@ typedef struct mysize {
 
 /** Defines checksum object */
 typedef struct mychecksum {
-  char     *md5;                   /**< A md5 checksum computed for the path */
+  char     *hash;                 /**< A checksum hash computed for the path */
+  int       type;                   /**< The type of hash (e.g. md5 or sha1) */
+  int       length;                                  /**< Length of the hash */
   int       action;                           /**< Action in case of failure */
   int       event_flag;         /**< TRUE if an event occured on this object */
   Command_T exec; /**< Optional command to be executed upon a checksum event */
@@ -503,6 +526,7 @@ typedef struct myservice {
   /** Common parameters */
   char *name;                                  /**< Service descriptive name */
   char *group;                                       /**< Service group name */
+  int (*check)(struct myservice *);       /**< Service verification function */
   Command_T start;                    /**< The start command for the service */
   Command_T stop;                      /**< The stop command for the service */
   int  type;                                     /**< Monitored service type */
@@ -521,49 +545,36 @@ typedef struct myservice {
   int  depend_visited;/**< Depend visited flag, set if dependencies are used */
   int  event_handled; /**< TRUE if an event has been handled on this service */
 
-  Checksum_T checksum;                                   /**< Checksum check */
+  Mail_T      maillist;                  /**< Alert notification mailinglist */
   Dependant_T dependantlist;                      /**<Dependant service list */
-  Device_T devicelist;                                /**< Device check list */
-  Gid_T gid;                                                  /**< Gid check */
-  Icmp_T icmplist;                                      /**< ICMP check list */
-  Mail_T maillist;                       /**< Alert notification mailinglist */
-  Perm_T perm;                                         /**< Permission check */
-  Port_T portlist;                   /**< Portnumbers the service listens on */
-  Resource_T resourcelist;                           /**< Resouce check list */
-  Size_T sizelist;                                      /**< Size check list */
+
+  /** Tests */
+  Checksum_T  checksum;                                  /**< Checksum check */
+  Device_T    devicelist;                             /**< Device check list */
+  Gid_T       gid;                                            /**< Gid check */
+  Icmp_T      icmplist;                                 /**< ICMP check list */
+  Perm_T      perm;                                    /**< Permission check */
+  Port_T      portlist;              /**< Portnumbers the service listens on */
+  Resource_T  resourcelist;                          /**< Resouce check list */
+  Size_T      sizelist;                                 /**< Size check list */
   Timestamp_T timestamplist;                       /**< Timestamp check list */
-  Uid_T uid;                                                  /**< Uid check */
+  Uid_T       uid;                                            /**< Uid check */
   
   /** Runtime parameters */
   DeviceInfo_T devinfo;                       /**< Data for the device check */
-  ProcInfo_T procinfo;                        /**< Data for the procfs check */
+  ProcInfo_T   procinfo;                      /**< Data for the procfs check */
 
   /** Context specific parameters */
   char *path;   /**< Path to the device, file, directory or process pid file */
   
   /** For internal use */
-  pthread_mutex_t mutex;          /**< Mutex used for action synchronization */
+  pthread_mutex_t   mutex;        /**< Mutex used for action synchronization */
   struct myservice *next;                         /**< next service in chain */
   struct myservice *next_conf;      /**< next service according to conf file */
   struct myservice *next_depend;           /**< next depend service in chain */
 } *Service_T;
 
 
-/** Defines process tree */
-typedef struct myprocesstree {
-  int  pid;
-  int  ppid;
-  long mem_kbyte;
-  int  children_num;
-  int  children_sum;
-  long mem_kbyte_sum;
-
-  /** For internal use */
-  struct myprocesstree *    parent;
-  struct myprocesstree **   children;
-} ProcessTree_T;
-
-
 /* -------------------------------------------------------- Global variables */
 
 
@@ -571,6 +582,8 @@ char   *prog;                                  /**< The Name of this Program */
 struct myrun Run;                      /**< Struct holding runtime constants */
 Service_T servicelist;                /**< The service list (created in p.y) */
 Service_T servicelist_conf;   /**< The service list in conf file (c. in p.y) */
+ProcessTree_T *ptree;                        /**< Global process information */
+int            ptreesize;                    /**< Global process information */
 
 
 /* ------------------------------------------------------- Public prototypes */
@@ -580,14 +593,15 @@ char *trim(char *);
 char *ltrim(char *);
 char *rtrim(char *);
 void  trim_quotes(char *);
+char *replace_char(char *, char, char);
 char *replace_string(char **, const char *, const char *);
 int   count_words(char *, const char *);
-int   starts_with(char *,char *);
+int   starts_with(const char *, const char *);
 void  handle_string_escapes(char *);
-void  chomp(char *);
-int   exist_service(char *);
+void  chomp(char *,int);
+int   exist_service(const char *);
 int   get_service_list_length();
-Service_T get_service(char *);
+Service_T get_service(const char *);
 void  printrunlist();
 void  printservice(Service_T);
 void  printservicelist();
@@ -600,32 +614,30 @@ int   exist_file(char *);
 char *get_RFC822date(long *);
 char *get_uptime(time_t delta, char *);
 char *get_process_uptime(char *, char *);
-char *get_md5sum(char *);
-int   check_md5(char *, char *);
+char *get_checksum(char *, int);
+int   check_hash(char *, char *, int);
 char *url_encode(char *uri);
+char *url_decode(char *url);
 char *get_basic_authentication_header();
 Auth_T get_user_credentials(char *);
 int   compare_user_credentials(char *, char *);
 int   parse(char *);
-void  control(char *);
-void  control_group(char *, char *);
-void  control_process(char *, char *);
-void  d_check_service(char *, char *);
-void  check_service(char *, char *);
+void  control(const char *);
+void  control_group(const char *, const char *);
+void  control_process(const char *, const char *);
+void  d_check_service(const char *, const char *);
+void  check_service(const char *, const char *);
 void  setup_dependants();
 void  reset_depend();
 void  spawn(Service_T, Command_T, const char *);
 void  status();
-void  status_group(char *);
 int   log_init() ;
 void  log_log(const char *format, ...) ;
 void  log_close();
 void  validate() ;
 void  daemonize() ;
 void  gc();
-void  gc_service(Service_T*);
 void  gc_mail_list(Mail_T*);
-void  gc_mail_server(MailServer_T*);
 void  init_files();
 time_t get_timestamp(char *, mode_t);
 void  finalize_files();
@@ -637,9 +649,11 @@ int   exist_daemon();
 void  sendmail(Mail_T);
 int   sock_msg(int sock, char *, ...);
 void  init_env();
+void  destroy_env();
 void *xmalloc (int);
 void *xcalloc(long, long);
 char *xstrdup(const char *);
+char *xstrndup(const char *, long);
 void *xresize(void *, long);
 void  monit_http(int);
 int   check_httpd();
@@ -652,12 +666,16 @@ int   check_file_stat(char *, char *, int);
 void  state_save();
 int   state_should_update();
 void  state_update();
-void  state_finalize();
 #if ! HAVE_MALLOC
 void *rpl_malloc (size_t __size);
 #endif
 void unset_signal_block(sigset_t *);
 void set_signal_block(sigset_t *, sigset_t *);
+int  check_process(Service_T);
+int  check_device(Service_T);
+int  check_file(Service_T);
+int  check_directory(Service_T);
+int  check_remote_host(Service_T);
 
 
 #endif
