@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -129,9 +129,9 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
- *  @author Martin Pala, <martin.pala@iol.cz>
+ *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.c,v 1.48 2003/10/27 10:46:25 hauk Exp $
+ *  @version \$Id: net.c,v 1.51 2004/02/16 22:42:32 hauk Exp $
  *
  *  @file
  */
@@ -276,7 +276,10 @@ int create_socket(const char *hostname, int port, int type, int timeout) {
   if(! set_noblock(s)) {
     goto error;
   }
-
+ 
+  if(fcntl(s, F_SETFD, FD_CLOEXEC) == -1)
+    goto error; 
+  
   if(do_connect(s, (struct sockaddr *)&sin, sizeof(sin), timeout) < 0) {
     goto error;
   }
@@ -398,7 +401,10 @@ int create_server_socket(int port, int backlog, const char *bindAddr) {
   
   if(! set_noblock(s))
     goto error;
-
+  
+  if(fcntl(s, F_SETFD, FD_CLOEXEC) == -1)
+    goto error; 
+  
   if(bind(s, (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in)) < 0)
     goto error;
   
