@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -98,8 +98,9 @@
 #include <sys/sysctl.h>
 #endif
 
+#include "monitor.h"
 #include "process.h"
-#include "sysdep.h"
+#include "process_sysdep.h"
 
 /**
  *  System dependent resource gathering code for MacOS X.
@@ -108,7 +109,7 @@
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Rory Toma <rory@digeo.com>
  *
- *  @version \$Id: sysdep_DARWIN.c,v 1.8 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: sysdep_DARWIN.c,v 1.11 2004/02/29 22:24:45 martinp Exp $
  *
  *  @file
  */
@@ -334,7 +335,6 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   /* Allocate the tree */
   
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
-  memset( pt, 0, sizeof(ProcessTree_T)*treesize );
 
   /* Insert data from /proc directory */
 
