@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -66,7 +66,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: sendmail.c,v 1.27 2003/10/25 19:17:11 hauk Exp $
+ *  @version \$Id: sendmail.c,v 1.29 2004/02/18 22:31:42 chopp Exp $
  *
  *  @file
  */
@@ -113,7 +113,7 @@ void sendmail(Mail_T mail) {
   open_server(&S);
   
   if(gethostname(S.localhost, sizeof(S.localhost)) < 0) {
-    snprintf(S.localhost, sizeof(S.localhost), LOCALHOST);
+    snprintf(S.localhost, sizeof(S.localhost), "%s", LOCALHOST);
   }
 
   /*
@@ -201,7 +201,7 @@ static void do_status(SendMail_T *S) {
     
   }
   
-  chomp(buf);
+  chomp(buf, STRLEN);
   
   sscanf(buf, "%d", &status);
   if(status >= 400) {
