@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -50,7 +50,7 @@
  * Implementation of the socket interface.
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- * @version \$Id: socket.c,v 1.32 2003/10/25 19:17:11 hauk Exp $
+ * @version \$Id: socket.c,v 1.34 2004/02/18 14:05:54 hauk Exp $
  * @file
  */
 
@@ -391,7 +391,7 @@ int socket_print(Socket_T S, const char *m, ...) {
   buf= format(m, ap, &l);
   va_end(ap);
   
-  n= socket_write(S, buf, strlen(buf));
+  n= socket_write(S, buf, l);
   FREE(buf);
   
   return n;
@@ -497,8 +497,7 @@ char *socket_readln(Socket_T S, char *s, int size) {
 
   /*
    * TODO: We should use an internal buffer and read a large chunk
-   * from the socket not just one by one byte, this is horribly
-   * ineffective.
+   * from the socket not just one by one byte.
    */
   while(--size && ((socket_read(S, s, 1)) > 0)) {
     if(*s++ == 10)
