@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -70,24 +70,19 @@
 #include "event.h"
 #include "socket.h"
 #include "net.h"
-#include "monit_device.h"
-#include "monit_process.h"
+#include "device.h"
+#include "process.h"
 
 
 /**
- *  This function contains the main process and check machinery for
- *  monit. The validate function check services in the service list
- *  to see if they are running and that they accepts connections if
- *  ports are defined. Not running services are restarted. This
- *  function also handles the various tests per service that are
- *  defined in the control file.
+ *  Implementation of validation engine
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Olivier Beyssac, <ob@r14.freenix.org> (check_skip)
- *  @author Martin Pala <martin.pala@iol.cz>
+ *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: validate.c,v 1.98 2003/10/25 19:17:11 hauk Exp $
+ *  @version \$Id: validate.c,v 1.109 2004/02/29 22:24:44 martinp Exp $
  *  @file
  */
 
@@ -95,40 +90,30 @@
 /* -------------------------------------------------------------- Prototypes */
 
 
-static int  check_skip(Service_T);
 static int  compare_value(int, int, int);
 static void reset_resource_counter(Service_T);
 
-static int  check_process(Service_T, ProcessTree_T *, int);
 static int  check_process_state(Service_T, char *);
 static int  check_process_resources(Service_T, Resource_T, char *);
 static int  check_process_connection(Service_T, Port_T, char *);
 
 static int  check_icmp_connection(Service_T, Icmp_T, char *);
 
-static int  check_device(Service_T);
 static int  check_device_resources(Service_T, Device_T, char *);
 
-static int  check_file(Service_T);
-
-static int  check_directory(Service_T);
-
-static int  check_remote_host(Service_T);
-
 static int  check_timestamps(Service_T);
 static int  check_timestamp_item(Service_T, Timestamp_T, char *);
 
 static int  check_size_item(Service_T, Size_T, unsigned long, char *);
 
 static int  check_timeout(Service_T);
+static int  check_skip(Service_T);
 
 static int  check_checksum(Service_T, char *);
 static int  check_perm(Service_T, mode_t, char *);
 static int  check_uid(Service_T, uid_t, char *);
 static int  check_gid(Service_T, gid_t, char *);
 
-static void do_validate(Service_T, ProcessTree_T *, int);
-static int  do_not_validate(Service_T);
 static int  eval_actions(int, Service_T, char *, char *, int);
 
 
@@ -136,329 +121,109 @@ static int  eval_actions(int, Service_T, char *, char *, int);
 
 
 /**
- * Walk the service-list and validate each service
+ *  This function contains the main check machinery for  monit. The
+ *  validate function check services in the service list to see if
+ *  they will pass all defined tests.
  */
 void validate() {
 
   Service_T s;
-  int treesize= 0;
-  ProcessTree_T *pt;
+  sigset_t ns, os;
 
-  if(Run.doprocess) {
-
-    treesize = initprocesstree(&pt);
-
-  }
-
-  if(! update_loadavg()) {
-    
+  if(! update_loadavg())
     log("Update of loadavg has failed!\n");
-    
-  }
-  
+
+  if(Run.doprocess)
+    initprocesstree();
+
   for(s= servicelist; s; s= s->next) {
     if(s->visited)
-	continue;
+      continue;
     LOCK(s->mutex)
-	do_validate(s, pt, treesize);
+      set_signal_block(&ns, &os);
+      if(s->do_monitor && !check_skip(s) && !check_timeout(s))
+        s->check(s);
+      unset_signal_block(&os);
     END_LOCK;
   }
 
+  if(Run.doprocess)
+    delprocesstree();
+
   reset_depend();
 
-  if (Run.doprocess) {
-
-    delprocesstree(pt, treesize);
-
-  }
-
-}
-
-
-/* --------------------------------------------------------------- Private */
-
-
-/**
- * Validate the given service - If the service is not running, start
- * it. If the service listens on a port try to connect to the port and
- * test the stated protocol at the port. If any of the various tests
- * fails then do the specified action on the service.
- */
-static void do_validate(Service_T s, ProcessTree_T * pt, int treesize) {
-  
-  sigset_t ns,os;
-
-  ASSERT(s);
-  
-  set_signal_block(&ns, &os);
-
-  /* First, check for pre-conditions */
-  if(do_not_validate(s)) {
-    goto reinstall;
-  }
-
-  /** Context specific service actions */
-  switch(s->type) {
-    
-  case TYPE_PROCESS:
-      if (! check_process(s, pt, treesize))
-	  goto reinstall;
-      break;
-      
-  case TYPE_DEVICE:
-      if (! check_device(s))
-	  goto reinstall;
-      break;
-
-  case TYPE_FILE:
-      if (! check_file(s))
-	  goto reinstall;
-      break;
-      
-  case TYPE_DIRECTORY:
-      if (! check_directory(s))
-	  goto reinstall;
-      break;
-      
-  case TYPE_REMOTE:
-      if (! check_remote_host(s))
-	  goto reinstall;
-      break;
-      
-  default:
-      break;
-      
-  }
-  
-  /** Common service actions */
-  if(! check_timestamps(s))
-      goto reinstall;
-  
-  reinstall:
-  unset_signal_block(&os);
-  
 }
 
 
 /**
- * Returns the logical OR of pre-condition tests.
- * A variant of the template-method pattern
+ * Validate a given process service s. Events are posted according to 
+ * its configuration. In case of a fatal event FALSE is returned.
  */
-static int do_not_validate(Service_T s) {
+int check_process(Service_T s) {
 
-  ASSERT(s);
+  pid_t  pid= -1;
+  Port_T pp= NULL;
+  Resource_T pr= NULL;
+  char report[STRLEN]={0};
   
-  return(!s->do_monitor   ||
-	 check_skip(s)    ||
-	 check_timeout(s));
-
-}
-
-
-/**
- * Evaluate actions of a service.  Post appropriate actions if necessary.
- * Return FALSE in case of a fatal event.
- */
-static int eval_actions(int action, Service_T s, char *report, char *check,
-			int event) {
-
-  ASSERT(s);
-  ASSERT(check);
-  ASSERT(report);
-
-  switch(action) {
-  case ACTION_ALERT:
-      Event_post(s, event, "Event: %s\n", report);
-      break; /* continue */
-    
-  case ACTION_STOP:
-      Event_post(s, EVENT_STOP, "Event: %s\n", report);
-      return FALSE;
-      
-  case ACTION_RESTART:
-      Event_post(s, EVENT_RESTART, "Event: %s\n", report);
-      return FALSE;
-      
-  case ACTION_EXEC:
-      Event_post(s, event, "Event: %s\n", report);
-      break; /* continue */
-
-  case ACTION_UNMONITOR:
-      Event_post(s, event, "Event: %s\n", report);
-      Event_post(s, EVENT_UNMONITOR, "Event: %s\n", report);
-      return FALSE;
-
-  default:
-      log("'%s' Unknow %s failure action.\n", s->name, check);
-      break;
-    
+  /* Test for running process */
+  if(!(pid= is_process_running(s))) {
+    /* Reset the proc info object to prevent false data in the first run */
+    memset(s->procinfo, 0, sizeof *(s->procinfo));
+    Event_post(s, EVENT_START, "Event: Process '%s' is not running.\n",
+	       s->name);
+    return FALSE;
+  } else {
+    DEBUG("'%s' is running with pid %d\n", s->name, (int)pid);
   }
 
-  return TRUE;
-}
-
-
-/**
- * Validate all given timestamps of a service s.  Events are posted according 
- * to its configuration.  In case of a fatal event FALSE is returned.
- */
-static int check_timestamps(Service_T s) {
-
-  Timestamp_T tl;
-  char report[STRLEN];
-  
-  for(tl= s->timestamplist; tl; tl= tl->next) {
-    if(!check_timestamp_item(s, tl, report)) {
-      tl->event_flag= TRUE; /* Turn on the object's event_flag to
-			     * indicate that the timestamp event
-			     * occured on this particular object */
-      if(! eval_actions(tl->action, s, report, "timestamp",
-			EVENT_TIMESTAMP)) {
-	return FALSE;
+  if(Run.doprocess) {
+    if(update_process_data(s, ptree, ptreesize, pid)) {
+      if(! check_process_state(s, report)) {
+	Event_post(s, EVENT_RESOURCE, "Event: %s\n", report);
+      } else {
+	DEBUG("'%s' check_process_state() passed.\n", s->name);
       }
+      
+      for(pr= s->resourcelist; pr; pr= pr->next) {
+	if(!check_process_resources(s, pr, report)) {
+	  pr->cycle=0;
+	  if(! pr->event_handled) {
+	    pr->event_flag= TRUE;
+	    pr->event_handled= TRUE;
+	    if(! eval_actions(pr->action, s, report, "resource",
+			      EVENT_RESOURCE)) {
+	      reset_resource_counter(s);
+	      return FALSE;
+	    }
+	  }
+	} else {
+	  pr->event_handled= FALSE;
+	}
+      }
+    } else {
+      log("'%s' failed to get service data\n", s->name);
     }
   }
-  
-  return TRUE;
 
-}
-
-
-/**
- * Validate a given directory service s.  Events are posted according to 
- * its configuration.  In case of a fatal event FALSE is returned.
- */
-static int check_directory(Service_T s) {
-
-  struct stat stat_buf;
-  char report[STRLEN]= {0};
-
-  if(stat(s->path, &stat_buf) != 0) {
-    if(! s->event_handled) {
-      Event_post(s, EVENT_START,
-		 "Event: directory '%s' doesn't exist\n", s->name);
-      s->event_handled= TRUE;
-    }
-    return FALSE;
-  } else {
-    s->event_handled= FALSE;
-  }
-
-  if(!S_ISDIR(stat_buf.st_mode)) {
-    if(!s->event_handled) {
-      Event_post(s, EVENT_UNMONITOR,
-		 "Event: '%s' is not directory\n", s->name);
-      s->event_handled= TRUE;
-    }
-    return FALSE;
-  } else {
-    s->event_handled= FALSE;
-  }
-  
-  if(check_perm(s, stat_buf.st_mode, report)) {
-    s->perm->event_flag= TRUE;
-    if(! eval_actions(s->perm->action, s, report, "permission",
-		      EVENT_PERMISSION))
-	return FALSE;
-  }
-  
-  if(check_uid(s, stat_buf.st_uid, report)) {
-    s->uid->event_flag= TRUE;
-    if(! eval_actions(s->uid->action, s, report, "uid", EVENT_UID))
-	return FALSE;
-  } 
-  
-  if(check_gid(s, stat_buf.st_gid, report)) {
-    s->gid->event_flag= TRUE;
-    if(! eval_actions(s->gid->action, s, report, "gid", EVENT_GID))
-	return FALSE;
-  } 
-
-  return TRUE;
-
-}
-
-
-/**
- * Validate a given file service s.  Events are posted according to 
- * its configuration.  In case of a fatal event FALSE is returned.
- */
-static int check_file(Service_T s) {
-
-  Size_T sl;
-  struct stat stat_buf;
-  char report[STRLEN]= {0};
-
-  if(stat(s->path, &stat_buf) != 0) {
-    if(! s->event_handled) {
-      Event_post(s, EVENT_START, "Event: file '%s' doesn't exist\n", s->name);
-      s->event_handled= TRUE;
-    }
-    return FALSE;
-  } else {
-    s->event_handled= TRUE;
-  }
-  
-  if(!S_ISREG(stat_buf.st_mode)) {
-    if(! s->event_handled) {
-      Event_post(s, EVENT_UNMONITOR,
-		 "Event: '%s' is not regular file\n", s->name);
-      s->event_handled= TRUE;
-    }
-    return FALSE;
-  } else {
-    s->event_handled= FALSE;
-  }
-  
-  if(check_checksum(s, report)) {
-      s->checksum->event_flag= TRUE;
-      if(! eval_actions(s->checksum->action, s, report, "checksum",
-			EVENT_CHECKSUM))
-	  return FALSE;
-  }
-  
-  if(check_perm(s, stat_buf.st_mode, report)) {
-    s->perm->event_flag= TRUE;
-    if(! eval_actions(s->perm->action, s, report, "permission",
-		      EVENT_PERMISSION))
-	return FALSE;
-  }
-
-  if(check_uid(s, stat_buf.st_uid, report)) {
-    s->uid->event_flag= TRUE;
-    if(! eval_actions(s->uid->action, s, report, "uid", EVENT_UID))
-	return FALSE;
-  }
-  
-  if(check_gid(s, stat_buf.st_gid, report)) {
-    s->gid->event_flag= TRUE;
-    if(! eval_actions(s->gid->action, s, report, "gid", EVENT_GID))
-	return FALSE;
-  }
-
-  for(sl= s->sizelist; sl; sl= sl->next) {
-    if(!check_size_item(s, sl, (unsigned long)stat_buf.st_size, report)) {
-      if(! sl->event_handled) {
-	/* Turn on the object's event_flag to indicate that the size event
-	 * occured on this particular object */
-	sl->event_flag= TRUE;
-	sl->event_handled= TRUE; /* Turn on the object's
-				  * event_handled flag so this
-				  * test is not handled in
-				  * subsequent poll cycles until
-				  * the error condition was reset.
-				  */
-	if(! eval_actions(sl->action, s, report, "size", EVENT_SIZE)) {
+  /* Test each host:port and protocol in the service's portlist */
+  for(pp= s->portlist; pp; pp= pp->next) {
+    if(!check_process_connection(s, pp, report)) {
+      if(! pp->event_handled) {
+	pp->event_flag= TRUE;
+	pp->event_handled= TRUE;
+	if(! eval_actions(pp->action, s, report, "connection",
+			  EVENT_CONNECTION)) {
 	  return FALSE;
 	}
       }
     } else {
-      sl->event_handled= FALSE;
+      pp->event_handled= FALSE;
     }
   }
-
+  
   return TRUE;
-
+  
 }
 
 
@@ -466,7 +231,7 @@ static int check_file(Service_T s) {
  * Validate a given device service s.  Events are posted according to 
  * its configuration.  In case of a fatal event FALSE is returned.
  */
-static int check_device(Service_T s) {
+int check_device(Service_T s) {
 
   Device_T td;
   struct stat stat_buf;
@@ -546,12 +311,156 @@ static int check_device(Service_T s) {
 }
 
 
+/**
+ * Validate a given file service s.  Events are posted according to 
+ * its configuration.  In case of a fatal event FALSE is returned.
+ */
+int check_file(Service_T s) {
+
+  Size_T sl;
+  struct stat stat_buf;
+  char report[STRLEN]= {0};
+
+  if(stat(s->path, &stat_buf) != 0) {
+    if(! s->event_handled) {
+      Event_post(s, EVENT_START, "Event: file '%s' doesn't exist\n", s->name);
+      s->event_handled= TRUE;
+    }
+    return FALSE;
+  } else {
+    s->event_handled= TRUE;
+  }
+  
+  if(!S_ISREG(stat_buf.st_mode)) {
+    if(! s->event_handled) {
+      Event_post(s, EVENT_UNMONITOR,
+		 "Event: '%s' is not regular file\n", s->name);
+      s->event_handled= TRUE;
+    }
+    return FALSE;
+  } else {
+    s->event_handled= FALSE;
+  }
+  
+  if(check_checksum(s, report)) {
+      s->checksum->event_flag= TRUE;
+      if(! eval_actions(s->checksum->action, s, report, "checksum",
+			EVENT_CHECKSUM))
+	  return FALSE;
+  }
+  
+  if(check_perm(s, stat_buf.st_mode, report)) {
+    s->perm->event_flag= TRUE;
+    if(! eval_actions(s->perm->action, s, report, "permission",
+		      EVENT_PERMISSION))
+	return FALSE;
+  }
+
+  if(check_uid(s, stat_buf.st_uid, report)) {
+    s->uid->event_flag= TRUE;
+    if(! eval_actions(s->uid->action, s, report, "uid", EVENT_UID))
+	return FALSE;
+  }
+  
+  if(check_gid(s, stat_buf.st_gid, report)) {
+    s->gid->event_flag= TRUE;
+    if(! eval_actions(s->gid->action, s, report, "gid", EVENT_GID))
+	return FALSE;
+  }
+
+  for(sl= s->sizelist; sl; sl= sl->next) {
+    if(!check_size_item(s, sl, (unsigned long)stat_buf.st_size, report)) {
+      if(! sl->event_handled) {
+	/* Turn on the object's event_flag to indicate that the size event
+	 * occured on this particular object */
+	sl->event_flag= TRUE;
+	sl->event_handled= TRUE; /* Turn on the object's
+				  * event_handled flag so this
+				  * test is not handled in
+				  * subsequent poll cycles until
+				  * the error condition was reset.
+				  */
+	if(! eval_actions(sl->action, s, report, "size", EVENT_SIZE)) {
+	  return FALSE;
+	}
+      }
+    } else {
+      sl->event_handled= FALSE;
+    }
+  }
+
+  if(!check_timestamps(s))
+    return FALSE;
+
+  return TRUE;
+
+}
+
+
+/**
+ * Validate a given directory service s.  Events are posted according to 
+ * its configuration.  In case of a fatal event FALSE is returned.
+ */
+int check_directory(Service_T s) {
+
+  struct stat stat_buf;
+  char report[STRLEN]= {0};
+
+  if(stat(s->path, &stat_buf) != 0) {
+    if(! s->event_handled) {
+      Event_post(s, EVENT_START,
+		 "Event: directory '%s' doesn't exist\n", s->name);
+      s->event_handled= TRUE;
+    }
+    return FALSE;
+  } else {
+    s->event_handled= FALSE;
+  }
+
+  if(!S_ISDIR(stat_buf.st_mode)) {
+    if(!s->event_handled) {
+      Event_post(s, EVENT_UNMONITOR,
+		 "Event: '%s' is not directory\n", s->name);
+      s->event_handled= TRUE;
+    }
+    return FALSE;
+  } else {
+    s->event_handled= FALSE;
+  }
+  
+  if(check_perm(s, stat_buf.st_mode, report)) {
+    s->perm->event_flag= TRUE;
+    if(! eval_actions(s->perm->action, s, report, "permission",
+		      EVENT_PERMISSION))
+	return FALSE;
+  }
+  
+  if(check_uid(s, stat_buf.st_uid, report)) {
+    s->uid->event_flag= TRUE;
+    if(! eval_actions(s->uid->action, s, report, "uid", EVENT_UID))
+	return FALSE;
+  } 
+  
+  if(check_gid(s, stat_buf.st_gid, report)) {
+    s->gid->event_flag= TRUE;
+    if(! eval_actions(s->gid->action, s, report, "gid", EVENT_GID))
+	return FALSE;
+  } 
+
+  if(!check_timestamps(s))
+    return FALSE;
+
+  return TRUE;
+
+}
+
+
 /**
  * Validate a remote service.
  * @param s The remote service to validate
  * @return FALSE if there was an error otherwise TRUE
  */
-static int check_remote_host(Service_T s) {
+int check_remote_host(Service_T s) {
 
   Port_T p= NULL;
   char report[STRLEN]={0};
@@ -608,75 +517,49 @@ static int check_remote_host(Service_T s) {
 }
 
 
+/* --------------------------------------------------------------- Private */
+
+
 /**
- * Validate a given process service s.  Events are posted according to 
- * its configuration.  In case of a fatal event FALSE is returned.
+ * Evaluate actions of a service.  Post appropriate actions if necessary.
+ * Return FALSE in case of a fatal event.
  */
-static int check_process(Service_T s, ProcessTree_T * pt, int treesize) {
+static int eval_actions(int action, Service_T s, char *report, char *check,
+			int event) {
 
-  pid_t  pid= -1;
-  Port_T pp= NULL;
-  Resource_T pr= NULL;
-  char report[STRLEN]={0};
-  
-  /* Test for running process */
-  if(!(pid= is_process_running(s))) {
-    /* Reset the proc info object to prevent false data in the first run */
-    memset(s->procinfo, 0, sizeof *(s->procinfo));
-    Event_post(s, EVENT_START, "Event: Process '%s' is not running.\n",
-	       s->name);
-    return FALSE;
-  } else {
-    DEBUG("'%s' is running with pid %d\n", s->name, (int)pid);
-  }
+  ASSERT(s);
+  ASSERT(check);
+  ASSERT(report);
 
-  if(Run.doprocess) {
-    if(update_process_data(s, pt, treesize, pid)) {
-      if(! check_process_state(s, report)) {
-	Event_post(s, EVENT_RESOURCE, "Event: %s\n", report);
-      } else {
-	DEBUG("'%s' check_process_state() passed.\n", s->name);
-      }
+  switch(action) {
+  case ACTION_ALERT:
+      Event_post(s, event, "Event: %s\n", report);
+      break; /* continue */
+    
+  case ACTION_STOP:
+      Event_post(s, EVENT_STOP, "Event: %s\n", report);
+      return FALSE;
       
-      for(pr= s->resourcelist; pr; pr= pr->next) {
-	if(!check_process_resources(s, pr, report)) {
-	  pr->cycle=0;
-	  if(! pr->event_handled) {
-	    pr->event_flag= TRUE;
-	    pr->event_handled= TRUE;
-	    if(! eval_actions(pr->action, s, report, "resource",
-			      EVENT_RESOURCE)) {
-	      reset_resource_counter(s);
-	      return FALSE;
-	    }
-	  }
-	} else {
-	  pr->event_handled= FALSE;
-	}
-      }
-    } else {
-      log("'%s' failed to get service data\n", s->name);
-    }
+  case ACTION_RESTART:
+      Event_post(s, EVENT_RESTART, "Event: %s\n", report);
+      return FALSE;
+      
+  case ACTION_EXEC:
+      Event_post(s, event, "Event: %s\n", report);
+      break; /* continue */
+
+  case ACTION_UNMONITOR:
+      Event_post(s, event, "Event: %s\n", report);
+      Event_post(s, EVENT_UNMONITOR, "Event: %s\n", report);
+      return FALSE;
+
+  default:
+      log("'%s' error -- unknown failure action: [%s]\n", s->name, check);
+      break;
+    
   }
 
-  /* Test each host:port and protocol in the service's portlist */
-  for(pp= s->portlist; pp; pp= pp->next) {
-    if(!check_process_connection(s, pp, report)) {
-      if(! pp->event_handled) {
-	pp->event_flag= TRUE;
-	pp->event_handled= TRUE;
-	if(! eval_actions(pp->action, s, report, "connection",
-			  EVENT_CONNECTION)) {
-	  return FALSE;
-	}
-      }
-    } else {
-      pp->event_handled= FALSE;
-    }
-  }
-  
   return TRUE;
-  
 }
 
 
@@ -766,17 +649,16 @@ static int check_process_state(Service_T s, char *report) {
   pi= s->procinfo;
   
   if(pi->status_flag & PROCESS_ZOMBIE) {
-    snprintf(report, STRLEN, 
-	     "process with pid %d is a zombie", pi->pid);
+    snprintf(report, STRLEN, "process with pid %d is a zombie", pi->pid);
     /* We do not check the process anymore if it's a zombie
        since such a process is (usually) unmanageable */
     LOCK(Run.mutex)
-	s->do_monitor= FALSE;
+      s->do_monitor= FALSE;
     END_LOCK;
     return FALSE;
   } else {
     DEBUG("'%s' zombie check passed [status_flag=%04x]\n",
-	  s->name,  pi->status_flag);
+          s->name,  pi->status_flag);
   }
   
   return TRUE;
@@ -926,7 +808,7 @@ static int check_process_resources(Service_T s, Resource_T pr, char *report) {
     break;
 
   default:
-      log("'%s' error: unknow resource ID: [%d]\n", s->name, pr->resource_id);
+      log("'%s' error -- unknown resource ID: [%d]\n", s->name, pr->resource_id);
   }
 
   if(okay && pr->cycle > 0) {
@@ -1021,12 +903,12 @@ static int check_checksum(Service_T s, char *report) {
   if(!s->checksum)
     return FALSE;
 
-  if( !check_md5(s->path, s->checksum->md5) ) {
+  if( !check_hash(s->path, s->checksum->hash, s->checksum->type) ) {
     snprintf(report, STRLEN, "checksum test failed for %s", s->path);
     /* Set the old checksum to the new value so we do not report this
      * more than once per change */
-    FREE(s->checksum->md5);
-    s->checksum->md5= get_md5sum(s->path);
+    FREE(s->checksum->hash);
+    s->checksum->hash= get_checksum(s->path, s->checksum->type);
     return TRUE;
   }
   
@@ -1124,6 +1006,32 @@ static int check_gid(Service_T s, gid_t gid, char *report) {
 }
 
 
+/**
+ * Validate all given timestamps of a service s.  Events are posted according 
+ * to its configuration.  In case of a fatal event FALSE is returned.
+ */
+static int check_timestamps(Service_T s) {
+
+  Timestamp_T tl;
+  char report[STRLEN];
+  
+  for(tl= s->timestamplist; tl; tl= tl->next) {
+    if(!check_timestamp_item(s, tl, report)) {
+      tl->event_flag= TRUE; /* Turn on the object's event_flag to
+			     * indicate that the timestamp event
+			     * occured on this particular object */
+      if(! eval_actions(tl->action, s, report, "timestamp",
+			EVENT_TIMESTAMP)) {
+	return FALSE;
+      }
+    }
+  }
+  
+  return TRUE;
+
+}
+
+
 /**
  * Returns TRUE if the timestamp test succeded, otherwise FALSE
  */
@@ -1291,7 +1199,7 @@ static int check_device_resources(Service_T t, Device_T td, char *report) {
       return TRUE;
       
   default:
-      log("'%s' error: unknow resource type: [%d]\n", t->name, td->resource);
+      log("'%s' error -- unknown resource type: [%d]\n", t->name, td->resource);
       return FALSE;
   }
   
@@ -1322,7 +1230,7 @@ static int compare_value(int operator, int left, int right) {
 	  return TRUE;
       break;
   default:
-      log("Unknow comparison operator\n");
+      log("Unknown comparison operator\n");
       return FALSE;
   }
 
