@@ -1,6 +1,6 @@
 /*
  * Copyright (C), 1998 by Eric S. Raymond.
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,7 +48,7 @@
  *  @author Eric S. Raymond <esr@snark.thyrsus.com>
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: xmalloc.c,v 1.9 2003/10/25 19:17:12 hauk Exp $
+ *  @version \$Id: xmalloc.c,v 1.11 2004/01/29 17:52:11 martinp Exp $
  *
  *  @file
  */
@@ -117,6 +117,21 @@ char *xstrdup(const char *s) {
 }
 
 
+char *xstrndup(const char *s, long l) {
+
+  char *t;
+
+  ASSERT(s);
+  
+  t= xmalloc(l + 1);
+  strncpy(t, s, l);
+  t[l]= 0;
+
+  return t;
+  
+}
+
+
 void *xresize(void *p, long nbytes) {
 
   if (p == 0) {
